package ai.nextbillion.octant.service;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Build;
import android.os.IBinder;
import android.text.TextUtils;

import java.lang.ref.WeakReference;

import ai.nextbillion.octant.Logger;
import io.flutter.plugin.common.MethodChannel;

public class OctantLocalConnection implements ServiceConnection {

    private boolean isBound = false;
    private WeakReference<Context> applicationContext;
    private OctantLocalService octantLocalService;
    private String userId;
    private String teamCode;

    public OctantLocalConnection(Context context) {
        this.applicationContext = new WeakReference<>(context.getApplicationContext());
        Logger.e("OctantLocalConnection", "Constructor");

    }

    public void setContext(Context context) {
        this.applicationContext = new WeakReference<>(context.getApplicationContext());
    }

    @Override
    public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
        Logger.e("OctantLocalConnection", "onServiceConnected");
        isBound = true;
        OctantLocalService.LocalBinder localBinder = (OctantLocalService.LocalBinder) iBinder;
        octantLocalService = localBinder.getService();
        octantLocalService.setUser(userId, teamCode);
        if (startServiceResult != null) {
            startServiceResult.success(true);
            startServiceResult = null;
        }
    }

    @Override
    public void onServiceDisconnected(ComponentName componentName) {
        Logger.e("OctantLocalConnection", "onServiceDisconnected");
        octantLocalService = null;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Interface
    ///////////////////////////////////////////////////////////////////////////

    public void init(String userId, String teamCode) {
        this.userId = userId;
        this.teamCode = teamCode;
        Logger.e("OctantLocalConnection", "OctantLocalService.isServiceRunning:" + OctantLocalService.isServiceRunning);
        if (OctantLocalService.isServiceRunning) {
            startService(null, null, null, null);
        }
    }

    public boolean updateUser(String userId, String teamCode) {
        if (octantLocalService != null) {
            octantLocalService.setUser(userId, teamCode);
            return true;
        }
        return false;
    }

    private MethodChannel.Result startServiceResult;

    public void startService(MethodChannel.Result result, String notificationDrawable, String notificationContent, String notificationTitle) {
        startServiceResult = result;
        if (!TextUtils.isEmpty(userId) && !TextUtils.isEmpty(teamCode)) {
            //        if (!isBound) {
            Context context = applicationContext.get();
            if (context != null) {
                Intent intent = getServiceIntent(context, notificationDrawable, notificationContent, notificationTitle);
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    context.startForegroundService(intent);
                } else {
                    context.startService(intent);
                }
                context.bindService(intent, this, Context.BIND_AUTO_CREATE);
            }
//        }
        } else {
            if (startServiceResult != null) {
                startServiceResult.error("Octant is not initialized", "startServiceResult", null);
                startServiceResult = null;
            }
//            throw new RuntimeException("userId and teamCode must not be null, have you called Octant.init(userId, teamCode) before calling startService()");
        }

    }

    public boolean startTracing() {
        if (octantLocalService != null) {
            octantLocalService.startTracing();
            return true;
        }
        return false;
    }

    public boolean stopTracing() {
        if (octantLocalService != null) {
            octantLocalService.stopTracing();
            return true;
        }
        return false;
    }

    public boolean stopService() {
        if (isBound) {
            Logger.e("LocalConnection", "stop service");
            Context context = applicationContext.get();
            if (context != null) {
                Logger.e("LocalConnection", "unbind service");
                context.unbindService(this);
                context.stopService(getServiceIntent(context, null, null, null));
            }
            octantLocalService = null;
            isBound = false;
            return true;
        }
        return false;

    }

    public boolean configureIntervals(long gatheringInterval, long uploadingInterval) {
        if (octantLocalService != null) {
            octantLocalService.configureIntervals(gatheringInterval, uploadingInterval);
            return true;
        }
        return false;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    ///////////////////////////////////////////////////////////////////////////

    private Intent getServiceIntent(Context context, String notificationDrawable, String notificationContent, String notificationTitle) {
        Intent intent = new Intent(context, OctantLocalService.class);
        if (!TextUtils.isEmpty(notificationDrawable)) {
            intent.putExtra("notificationDrawable", notificationDrawable);
        }
        if (!TextUtils.isEmpty(notificationContent)) {
            intent.putExtra("notificationContent", notificationContent);
        }
        if (!TextUtils.isEmpty(notificationTitle)) {
            intent.putExtra("notificationTitle", notificationTitle);
        }
        return intent;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    ///////////////////////////////////////////////////////////////////////////


    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getTeamCode() {
        return teamCode;
    }

    public void setTeamCode(String teamCode) {
        this.teamCode = teamCode;
    }

    public void unbindService() {
        Context context = applicationContext.get();
        if (context != null) {
            Logger.e("LocalConnection", "unbind service");
            context.unbindService(this);
        }
    }
}
