package ai.nextbillion.octant.service;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Build;
import android.os.IBinder;
import android.os.RemoteException;

import ai.nextbillion.octant.IOctantServiceInterface;

public class OctantRemoteConnection implements ServiceConnection {

    private boolean isBound = false;
    private final Context applicationContext;

    public OctantRemoteConnection(Context context) {
        this.applicationContext = context.getApplicationContext();
    }

    public void startService() {
        if (!isBound) {
            Intent intent = getServiceIntent();
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                applicationContext.startForegroundService(intent);
            } else {
                applicationContext.startService(intent);
            }
            applicationContext.bindService(intent, this, Context.BIND_AUTO_CREATE);

        }
    }

    public void stopService() {
        if (octantService != null) {
            try {
                octantService.stopTracing();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
        applicationContext.unbindService(this);
        isBound = false;
    }

    private Intent getServiceIntent() {
        return new Intent(applicationContext, OctantRemoteService.class);
    }

    ///////////////////////////////////////////////////////////////////////////
    // Binder
    ///////////////////////////////////////////////////////////////////////////

    IOctantServiceInterface octantService;

    @Override
    public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
        octantService = IOctantServiceInterface.Stub.asInterface(iBinder);
        isBound = true;
        try {
            octantService.startTracing();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onServiceDisconnected(ComponentName componentName) {
        octantService = null;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    ///////////////////////////////////////////////////////////////////////////

    public void configureFrequencies(long gatheringInterval, long uploadingInterval) {
        if (octantService != null) {
            try {
                octantService.configureIntervals(gatheringInterval, uploadingInterval);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    }

}
