package ai.nextbillion.octant.service;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.RemoteException;

import androidx.annotation.NonNull;
import androidx.core.app.NotificationCompat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import ai.nextbillion.octant.IOctantServiceInterface;
import ai.nextbillion.octant.Logger;
import ai.nextbillion.octant.R;
import ai.nextbillion.octant.engine.LocationEngine;
import ai.nextbillion.octant.engine.LocationEngineCallback;
import ai.nextbillion.octant.engine.LocationEngineProvider;
import ai.nextbillion.octant.engine.LocationEngineRequest;
import ai.nextbillion.octant.engine.LocationEngineResult;

public class OctantRemoteService extends Service {
//    private final IBinder localBinder = new LocalBinder();

    @Override
    public void onCreate() {
        Logger.e("OctantReportingService", "onCreate");
        super.onCreate();

    }

    @Override
    public IBinder onBind(Intent intent) {
        Logger.e("OctantReportingService", "onBind");
//        return localBinder;
        return new OctantServiceImpl();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Logger.e("OctantReportingService", "onStartCommand");
        createNotificationChannel(this);

        return START_STICKY;
    }


    @Override
    public void onDestroy() {
        Logger.e("OctantReportingService", "onDestroy");
        stopForeground(true);
        super.onDestroy();
    }

    ///////////////////////////////////////////////////////////////////////////
    // Notification
    ///////////////////////////////////////////////////////////////////////////

    public static final int REPORTING_NOTIFICATION_ID = 6501;
    public static final String REPORTING_NOTIFICATION_CHANNEL = "REPORTING_NOTIFICATION_CHANNEL";

    public void startLocationReporting() {
        Logger.e("OctantReportingService", "startLocationReporting");
        startForeground(REPORTING_NOTIFICATION_ID, buildNotification());
        activeLocationEngine(currentGatheringInterval);
        startTimer(currentUploadingInterval);
    }

    public class LocalBinder extends Binder {

        public OctantRemoteService getService() {
            return OctantRemoteService.this;
        }

    }

    Notification buildNotification() {
        PendingIntent pendingOpenIntent = createPendingOpenIntent(this);
        NotificationCompat.Builder builder = new NotificationCompat.Builder(getApplicationContext(), REPORTING_NOTIFICATION_CHANNEL)
                .setContentIntent(pendingOpenIntent)
                .setPriority(NotificationCompat.PRIORITY_MAX)
                .setCategory(NotificationCompat.CATEGORY_SERVICE)
                .setSmallIcon(R.drawable.ic_nb_icon)
                .setOngoing(true);
        return builder.build();
    }

    private void createNotificationChannel(Context context) {
        NotificationManager notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel notificationChannel = new NotificationChannel(
                    REPORTING_NOTIFICATION_CHANNEL, context.getString(R.string.channel_name),
                    NotificationManager.IMPORTANCE_LOW);
            notificationManager.createNotificationChannel(notificationChannel);
        }
    }

    private PendingIntent createPendingOpenIntent(Context context) {
        PackageManager pm = context.getPackageManager();
        Intent intent = pm.getLaunchIntentForPackage(context.getPackageName());
        intent.setPackage(null);
        return PendingIntent.getActivity(context, 0, intent, 0);
    }

    ///////////////////////////////////////////////////////////////////////////
    // Timer
    ///////////////////////////////////////////////////////////////////////////

    private Handler handler = new Handler();
    private Timer timer;

    private void initTimer() {

    }

    private void startTimer(long interval) {
        if (timer != null) {
            timer.cancel();
            timer.purge();
        }
        timer = new Timer();
        timer.scheduleAtFixedRate(new UploadingTimerTask(), interval, interval);
    }

    final class UploadingTimerTask extends TimerTask {

        @Override
        public void run() {
            uploadLocations();

            handler.post(new Runnable() {
                @Override
                public void run() {

                }
            });
        }
    }


    ///////////////////////////////////////////////////////////////////////////
    //
    ///////////////////////////////////////////////////////////////////////////
    // Default interval between location updates
    static final long DEFAULT_INTERVAL_MILLIS = 5 * 1000;
    static final long DEFAULT_UPLOADING_INTERVAL_MILLIS = 25 * 1000;

    List<Location> cachedLocations = Collections.synchronizedList(new ArrayList<Location>());

    private void uploadLocations() {
        List<Location> locationsToBeUploaded = new ArrayList<>(cachedLocations);
        cachedLocations.clear();
        Logger.e("uploadLocations", "uploading...:" + locationsToBeUploaded);
    }

    private LocationEngine locationEngine;


    private long currentGatheringInterval = DEFAULT_INTERVAL_MILLIS;
    private long currentUploadingInterval = DEFAULT_UPLOADING_INTERVAL_MILLIS;

    public void configureIntervals(long gatheringInterval, long uploadingInterval) {
        if (currentGatheringInterval != gatheringInterval) {
            currentGatheringInterval = gatheringInterval;
            activeLocationEngine(currentGatheringInterval);
        }

        if (currentUploadingInterval != uploadingInterval) {
            currentUploadingInterval = uploadingInterval;
            startTimer(currentUploadingInterval);
        }


    }

    private void activeLocationEngine(long interval) {
        if (locationEngine == null) {
            locationEngine = LocationEngineProvider.getBestLocationEngine(getApplicationContext());
        } else {
            locationEngine.removeLocationUpdates(locationEngineCallback);
        }
        locationEngine.requestLocationUpdates(buildEngineRequest(interval), locationEngineCallback, null);
        locationEngine.getLastLocation(locationEngineCallback);
    }


    private final LocationEngineCallback<LocationEngineResult> locationEngineCallback = new LocationEngineCallback<LocationEngineResult>() {
        @Override
        public void onSuccess(LocationEngineResult result) {
            Location location = result.getLastLocation();
            if (location != null) {
                Logger.e("LocationEngine", "onLocationFound:" + location.toString());
                cachedLocations.add(location);
            }

        }

        @Override
        public void onFailure(@NonNull Exception exception) {

        }
    };

    private LocationEngineRequest buildEngineRequest(long interval) {
        return new LocationEngineRequest.Builder(interval)
                .setFastestInterval(interval)
                .setPriority(LocationEngineRequest.PRIORITY_HIGH_ACCURACY)
                .build();
    }

    ///////////////////////////////////////////////////////////////////////////
    // Binder
    ///////////////////////////////////////////////////////////////////////////


    public class OctantServiceImpl extends IOctantServiceInterface.Stub {

        @Override
        public void startTracing() throws RemoteException {
            Logger.e("OctantServiceImpl", "startTracing");
            startLocationReporting();
        }

        @Override
        public void stopTracing() throws RemoteException {
            Logger.e("OctantServiceImpl", "stopTracing");
        }

        @Override
        public void configureIntervals(long gatheringInterval, long uploadingInterval) throws RemoteException {
            Logger.e("OctantServiceImpl", "configureIntervals");
        }
    }
}
