import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:octant/models/user_coords_response.dart';
import 'package:octant/models/user_rank.dart';

class Octant {
  static const String methodChannelInit = 'init';
  static const String methodChannelUpdateUserDetails = 'updateUserDetails';
  static const String methodChannelStartService = 'startService';
  static const String methodChannelStartTracing = 'startTracing';
  static const String methodChannelStopTracing = 'stopTracing';
  static const String methodChannelStopService = 'stopService';
  static const String methodChannelConfigureIntervals = 'configureIntervals';

  static const String methodChannelQueryRanking = 'queryRanking';
  static const String methodChannelQueryUserCoordinate = 'queryUserCoordinate';

  static const String methodChannelEnableDebug = 'enableDebug';

  static const String argumentBaseUrl = 'baseUrl';
  static const String argumentUserCode = 'userCode';
  static const String argumentTeamCode = 'teamCode';

  static const String argumentUserCodes = 'userCodes';
  static const String argumentStart = 'start';
  static const String argumentEnd = 'end';
  static const String argumentLimit = 'limit';
  static const String argumentRemarkCode = 'remarkCode';

  static const String argumentGatheringInterval = 'gatheringInterval';
  static const String argumentUploadingInterval = 'uploadingInterval';
  static const String argumentNotificationIconId = 'notificationIconId';

  static const String argumentNotificationTitle = "notificationTitle";
  static const String argumentNotificationContent = "notificationContent";

  static const String argumentEnableDebug = 'enableDebug';

  static const MethodChannel _channel = MethodChannel('octant');
  static String? userId;
  static String? teamCode;

  static Future init(String userId, String teamCode, [String? baseUrl]) async {
    Octant.userId = userId;
    Octant.teamCode = teamCode;
    await _channel.invokeMethod(methodChannelInit,
        {
          argumentUserCode: userId,
          argumentTeamCode: teamCode,
          argumentBaseUrl: baseUrl
        });
  }

  static Future updateUserDetails(String userId, String teamCode) async {
    Octant.userId = userId;
    Octant.teamCode = teamCode;
    bool? isServiceBound = await _channel.invokeMethod(
        methodChannelUpdateUserDetails,
        {argumentUserCode: userId, argumentTeamCode: teamCode});
    if (isServiceBound ?? false) {
      return Future.value(isServiceBound);
    } else {
      return Future.error('OctantService is not started or registered');
    }
  }

  static Future<bool?> startService(String notificationDrawable,
      String notificationTitle,
      String notificationContent) async {
    if (userId != null && teamCode != null) {
      bool? succeed = await _channel
          .invokeMethod<bool>(methodChannelStartService,
          {
            argumentNotificationIconId: notificationDrawable,
            argumentNotificationTitle: notificationTitle,
            argumentNotificationContent: notificationContent
          })
          .timeout(const Duration(seconds: 5), onTimeout: () {
        return false
      });
      return succeed;
    }
    return Future.error(
        'userId and teamCode must not be null, have you called Octant.init(userId, teamCode) before calling startService()');
  }

  static Future startTracing() async {
    bool? isServiceBound =
    await _channel.invokeMethod(methodChannelStartTracing);
    if (isServiceBound ?? false) {
      return Future.value(isServiceBound);
    } else {
      return Future.error('OctantService is not started or registered');
    }
  }

  static Future stopService() async {
    bool? isServiceBound =
    await _channel.invokeMethod(methodChannelStopService);
    if (isServiceBound ?? false) {
      return Future.value(isServiceBound);
    } else {
      return Future.error('OctantService is not started or registered');
    }
  }

  static Future configureIntervals(int gatheringFrequency,
      int uploadingFrequency) async {
    bool? isServiceBound =
    await _channel.invokeMethod(methodChannelConfigureIntervals, {
      argumentGatheringInterval: gatheringFrequency,
      argumentUploadingInterval: uploadingFrequency
    });
    if (isServiceBound ?? false) {
      return Future.value(isServiceBound);
    } else {
      return Future.error('OctantService is not started or registered');
    }
  }

  static Future<List<UserRank>> queryRanking(int startTime, int endTime,
      int limit,
      List<String> userCodeList, String remarkCode) async {
    print('querey ranking');
    String userCodes = userCodeList.join(',');
    List<dynamic>? ranks = await _channel.invokeMethod(
        methodChannelQueryRanking, {
      argumentStart: startTime.toString(),
      argumentEnd: endTime.toString(),
      argumentLimit: limit.toString(),
      argumentRemarkCode: remarkCode,
      argumentUserCodes: userCodes
    });
    print('queryRanking rsp ranks:$ranks');
    List<UserRank> rankList = ranks?.map((e) => UserRank.fromJson(e))
        .toList() ?? <UserRank>[];

    print('queryRanking rsp:$rankList');
    return rankList;
  }

  static Future queryUserCoordinate(int startTime, int endTime, int limit,
      String remarkCode) async {
    print('querey user coords');
    String? jsonString = await _channel.invokeMethod(
        methodChannelQueryUserCoordinate, {
      argumentStart: startTime.toString(),
      argumentEnd: endTime.toString(),
      argumentLimit: limit.toString(),
      argumentRemarkCode: remarkCode,
    });

    String source = jsonString ?? "";
    Map<String, dynamic>? jsonMap = json.decode(source);
    print('queryUserCoordinate rsp jsonMap:$jsonMap');
    UserCoordinateResponse userCoordinateResponse = UserCoordinateResponse
        .fromJson(jsonMap);
    print('queryUserCoordinate rsp:$userCoordinateResponse');
    return
      userCoordinateResponse;
  }

  static enableDebug(bool debug) async {
    await _channel.invokeMethod(
        methodChannelEnableDebug, {argumentEnableDebug: debug});
  }
}
